﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2013 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using HeuristicLab.Algorithms.GeneticAlgorithm;
using HeuristicLab.Problems.GPDL;
using HeuristicLab.Problems.GPDL.CodeGen;
using HeuristicLab.Problems.GPDL.Views;
using HeuristicLab.SequentialEngine;
using Microsoft.VisualStudio.TestTools.UnitTesting;

namespace Test {
  [TestClass]
  public class TestGpdlExamples {
    [TestMethod]
    public void TestExamples() {
      var prov = new GpdlExampleProvider();
      foreach (GpdlExampleDataDescriptor dd in prov.GetDataDescriptors()) {
        Scanner scanner = new Scanner(new MemoryStream(Encoding.UTF8.GetBytes(dd.Source)));
        Parser parser = new Parser(scanner);
        parser.Parse();
        Assert.IsNotNull(parser.AbstractSyntaxTree);

        // test: run one generation
        var ga = new GeneticAlgorithm();
        ga.Engine = new SequentialEngine();
        ga.PopulationSize.Value = 100;
        ga.MaximumGenerations.Value = 1;

        var problemGenerator = new ProblemGenerator();
        ga.Problem = problemGenerator.GenerateFromAst(parser.AbstractSyntaxTree);

        var wh = new AutoResetEvent(false);
        Exception e = null;
        ga.ExceptionOccurred += (sender, args) => {
          e = args.Value;
          wh.Set();
        };
        ga.Stopped += (sender, args) => { wh.Set(); };
        ga.Start();
        wh.WaitOne();
        if (e != null) Assert.Fail(e.Message, e);
      }
    }

  }
}
